* Call include file for the working directory and the survey name.
include "surveyname.sps".

get file = "hh.sav".

* select completed households.
select if (HH9 = 1).

compute tot = 1.
variable labels tot "Total".
value labels tot 1 " ".

compute total = 1.
variable labels total " ".
value labels total 1 "Number of households".
		
recode SL1 (0, sysmis = 1) (1 = 2) (2 thru 96 = 3) (97, 98, 99 = 9) into nch.
variable labels nch "Percent distribution of households by number of children age 1-17".
value labels nch
1 "None"
2 "1"
3 "2 or more"
9 "Missing".

* select households with 2 or more children.
do if (nch = 3).
+ compute sel = 0.
+ if (CD_FLAG =  1) sel = 100.
end if.

variable labels sel "Percent of households where correct selection was performed".

save outfile = 'tmp.sav'.

aggregate 
  /outfile = "tmp1.sav"
  /break = team
  /teamt = mean (sel).

aggregate 
  /outfile = "tmp2.sav"
  /break = tot
  /teamt = mean (sel).

* Add this summary information .
get file = 'tmp.sav'.
add files
  /file = *
  /file = 'tmp1.sav'
  /file = 'tmp2.sav'.

* target is set to be 95%. 
do if (teamt < 96).
+ compute target = teamt.
end if.

* to calculate correct number of households with at least two children exclude temporarily generated cases needed for calculation of target reached.
do if (sysmis (target) and nch = 3).
+ compute hhwch = 1.
end if.

variable labels target "Target not met".
variable labels hhwch " ".
value labels hhwch 1 "Number of households with 2 or more children age 1-17 years".

ctables
  /vlabels variables= total hhwch display=none
  /table tot [c] + team [c] by 
    nch [c] [rowpct.validn, '',f5.1] + total [c] [count, '',f5.0] + sel [s] [mean, '',f5.1] + hhwch [c] [count, '',f5.0] + target [s] [mean, '',f5.1] 
  /categories var=nch empty=exclude missing=exclude total = yes label = "Total"
  /slabels position=column visible = no
  /titles title = 
   "Percent distribution of households by number of children age 1-17 years, and percentage of households where correct selection for the child  labour "
   "and child discipine modules was performed, " + surveyname.	
	
new file.

erase file = "tmp.sav".		
erase file = "tmp1.sav".	
erase file = "tmp2.sav".		
		

