
PROC GLOBAL

set explicit

numeric minalt maxalt minlat maxlat minlong maxlong xalt xlat xlong mindi cln dd err ss;

PROC GPSX_FF
preproc
{ !!! UPDATE THESE VALUES FOR YOUR COUNTRY !!! }
minalt = -100;      { minimum altitude in the country}
maxalt = 4000;      { maximum altitude in the country}
minlat = -20;       { minimum latitude in the country - negative = South, positive = North }
maxlat = 50;        { maximum latitude in the country - negative = South, positive = North }
minlong = -20;      { minimum longitude in the country - negative = West, positive = East }
maxlong = 50;       { maximum longitude in the country - negative = West, positive = East }
mindi = 20091001;   { earliest interview date in yyyymmdd format } 

PROC GP1
{ check cluster is in the sample file }
recode GP1      => err;
       001-800  => 0;    { Give list of valid cluster numbers, both here and in the dcf for GP1 }
                => 1;
endrecode;
if err then
    {+EN} display("Cluster number not valid"); {EN+}
    {{ES} display("El numero del conglomerado no es valido"); {ES}}
    {{FR} display("Numero de grappe pas valide"); {FR}}
  reenter;
endif;

PROC GP2
{ !!! ENTER CORRESPONDENCE BETWEEN CLUSTER AND AREA FOR YOUR COUNTRY !!! }
recode GP1  : GP2 => err;
    001-100 :  1  => 0;
    101-200 :  2  => 0;
    201-300 :  1  => 0;
    301-400 :  2  => 0;
    401-500 :  1  => 0;
    501-600 :  2  => 0;
    601-700 :  1  => 0;
    701-800 :  2  => 0;
            :     => 1;
endrecode;
if err then
   {+EN} display("Cluster identification is incorrect"); {EN+}
   {{ES} display("La identificacion del conglomerado es incorrecta"); {ES}}
   {{FR} display("Numero d'identification incorrect"); {FR}}
  reenter
endif;

PROC GP3
{ !!! ENTER CORRESPONDENCE BETWEEN CLUSTER AND REGION FOR YOUR COUNTRY !!! }
recode GP1  : GP3 => err;
    001-100 :  1  => 0;
    101-200 :  1  => 0;
    201-300 :  2  => 0;
    301-400 :  2  => 0;
    401-500 :  3  => 0;
    501-600 :  3  => 0;
    601-700 :  4  => 0;
    701-800 :  4  => 0;
            :     => 1;
endrecode;
if err then
  {+EN} display("Cluster identification is incorrect"); {EN+}
  {{ES} display("La identificacion del conglomerado es incorrecta"); {ES}}
  {{FR} display("Numero d'identification incorrect"); {FR}}
  reenter
endif;

PROC GP5Y
{ !!! revise the date of GPS collection to reflect the correct time period }
recode GP5Y  :     GP5M        : GP5D => err;
   2011-2012 : 1,3,5,7,8,10,12 : 1-31 => 0;
   2011-2012 : 4,6,9,11        : 1-30 => 0;
   2011-2012 : 2               : 1-29 => 0;
             :                 :      => 1;
endrecode;
if err then
   {+EN} display("Date of interview impossible"); {EN+}
   {{ES} display("La fecha de la entrevista no es posible"); {ES}}
   {{FR} display("Date d'interview impossible"); {FR}}
  reenter
endif;
dd = (GP5Y*100 + GP5M)*100 + GP5D;
ss = sysdate("yyyymmdd");
if dd >= ss or dd < mindi then
  {+EN} display("Date of interview impossible"); {EN+}
  {{ES} display("La fecha de la entrevista no es posible"); {ES}}
  {{FR} display("Date d'interview impossible"); {FR}}
  reenter
endif;

PROC GP7A
xalt = GP7A;
if not xalt in minalt:maxalt then
    {+EN} display("The waypoint's altitude is outside the acceptable range (%dS:%dS)",minalt,maxalt);{EN+}
    {{ES} display("La altitud de la estacion esta fuera del rango aceptable (%dS:%dS)",minalt,maxalt);{ES}}
    {{FR} display("Le point de la altitude est hors des limites acceptees (%dS:%dS)",minalt,maxalt); {FR}}
endif;
PROC GP8NS
if GP8NS <> "N" & GP8NS <> "S" then
    {+EN} display("You must enter N for North or S for South."); {EN+}
    {{ES} display("Usted debe ingresar N por Norte o S por Sur."); {ES}}
    {{FR} display("Vous devez entrer N for Nord or S for Sud."); {FR}}
  reenter;
endif;

PROC GP8D
if GP8NS = "N" then
  xlat = GP8D;
elseif GP8NS = "S" then
  xlat = (0-GP8D);
endif;  
if not xlat in minlat:maxlat then
  if maxlat < 0 then
    {+EN} display("The waypoint's latitude is outside the acceptable range (%dS:%dS)",(0-minlat),(0-maxlat)); {EN+}    
    {{ES} display("La latitud de la estacion esta fuera del rango aceptable (%dS:%dS)",(0-minlat),(0-maxlat)); {ES}}
    {{FR} display("Le point de repere de la latitude est hors des limites acceptees (%dS:%dS)",(0-minlat),(0-maxlat)); {FR}}
  elseif minlat < 0 then
    {+EN} display("The waypoint's latitude is outside the acceptable range (%dS:%dN)",(0-minlat),maxlat);{EN+}    
    {{ES} display("La latitud de la estacion esta fuera del rango aceptable (%dS:%dS)",(0-minlat),(0-maxlat)); {ES}}
    {{FR}display("Le point de repere de la latitude est hors des limites acceptees (%dS:%dN)",(0-minlat),maxlat);{FR}}
  else  
    {+EN} display("The waypoint's latitude is outside the acceptable range (%dN:%dN)",minlat,maxlat);{EN+}    
    {{ES} display("La latitud de la estacion esta fuera del rango aceptable (%dS:%dS)",(0-minlat),(0-maxlat)); {ES}}  
    {{FR} display("Le point de repere de la latitude est hors des limites acceptees (%dN:%dN)",minlat,maxlat);{FR}}
  endif;
endif;

PROC GP9EW
if GP9EW <> "E" and GP9EW <> "W" and GP9EW <> "O"  then
    {+EN} display("You must enter E for East or W for West."); {EN+}
    {{ES} display("Usted debe ingresar E por Este o O por Oeste."); {ES}}
    {{FR} display("Vous devez entrer E for Est ou O for Ouest."); {FR}}
  reenter;
endif;

PROC GP9D
if GP9EW = "E" then
  xlong = GP9D;
elseif GP9EW = "W" or GP9EW = "O" then
  xlong = (0-GP9D);
endif;  
if not xlong in minlong:maxlong then
  if maxlong < 0 then
    {+EN} display("The waypoint's longitude is outside the acceptable range (%dW:%dW)",(0-minlong),(0-maxlong));{EN+}
    {{ES} display("La longitud de la estacion esta fuera del rango aceptable (%dW:%dW)",(0-minlong),(0-maxlong));{ES}}
    {{FR} display("Le point de repere de la longitude est hors des limites acceptees (%dW:%dW)",(0-minlong),(0-maxlong)); {FR}}
  elseif minlong < 0 then
    {+EN}display("The waypoint's longitude is outside the acceptable range (%dW:%dE)",(0-minlong),maxlong); {EN+}
    {{ES}display("La longitud de la estacion esta fuera del rango aceptable (%dW:%dE)",(0-minlong),maxlong); {ES}}
    {{FR}display("Le point de repere de la longitude est hors des limites acceptees (%dW:%dE)",(0-minlong),maxlong); {FR}}
  else  
    {+EN} display("The waypoint's longitude is outside the acceptable range (%dE:%dE)",minlong,maxlong);{EN+}
    {+EN} display("La longitud de la estacion esta fuera del rango aceptable (%dE:%dE)",minlong,maxlong); {EN+}
    {{FR} display("Le point de repere de la longitude est hors des limites acceptees (%dE:%dE)",minlong,maxlong); {FR}}
  endif;
endif;

PROC DP6
if DP6 <> GP6 then
  {+EN}display("Different waypoint names entered! Check entered values against the questionnaire");{EN+}
  {{ES}display("Los nombres de las estaciones ingresados son diferentes! Verifique los valores entrados con el cuestionario");{ES}}
  {{FR}display("Differents noms du point de repere saisis! Verifier les valeurs saisies par rapport aux donnees du questionnaire");{FR}}
  DP6 = "";
  reenter;
endif;

PROC DP7A
if DP7A <> GP7A then
  {+EN} display("Different altitudinal orientations entered! Check entered values against the questionnaire");{EN+}
  {{ES} display("Las orientaciones altitudinales son diferentes! Verifique los valores ingresados con el cuestionario");{ES}}
  {{FR} display("Differents altitudes saisis! Verifier les valeurs saisies par rapport aux donnees du questionnaire");{FR}}
  DP7A = 0;
  reenter;
endif;
PROC DP8NS
if DP8NS <> GP8NS then
  {+EN} display("Different latitudinal orientations entered! Check entered values against the questionnaire");{EN+}
  {{ES} display("Las orientaciones latitudinales ingresadas son diferentes! Verifique los valores ingresados con el cuestionario");{ES}}
  {{FR} display("Differents orientations latitudinales saisies! Verifier les valeurs saisies par rapport aux donnees du questionnaire"); {FR}}
  DP8NS = "";
  reenter;
endif;

PROC DP8D
if DP8D <> GP8D then
  {+EN} display("Different latitudinal decimal degrees entered! Check entered values against the questionnaire");{EN+}
  {{ES} display("Los grados decimales latitudinales ingresados son diferentes! Verifique los valores ingresados con el cuestionario");{ES}}
  {{FR} display("Differents degres latitudinaux saisis! Verifier les valeurs saisies par rapport aux donnees du questionnaires");{FR}}
  DP8D = 0;
  reenter;
endif;

PROC DP9EW
if DP9EW <> GP9EW then
  {+EN} display("Different longitudinal orientations entered! Check entered values against the questionnaire");{EN+}
  {{ES} display("Las orientaciones longitudinales ingresadas son diferentes! Verifique los valores ingresados con el cuestionario");{ES}}
  {{FR} display("Differentes orientations longitudinales saisies! Verifier les valeurs saisies par rapport aux donnees du questionnaire");{FR}}
  DP9EW = "";
  reenter;
endif;

PROC DP9D
if DP9D <> GP9D then
  {+EN} display("Different longitudinal decimal degrees entered! Check entered values against the questionnaire");{EN+}
  {{ES} display("Los grados decimales longitudinales ingresados son diferentes! Verifique los valores ingresados con el cuestionario");{ES}}
  {{FR} display("Differents degres lonngitudinaux saisis! Verifier les valeurs saisies par rapport aux donnees du questionnaires");{FR}}
  DP9D = 0;
  reenter;
endif;

